@@ -59,6 +59,17 @@ dwell_restore_single_click (MTClosure *mt)
     mt_service_set_clicktype (mt->service, DWELL_CLICK_TYPE_SINGLE, NULL);
 }
 
+static gboolean
+dwell_b1r_timeout (gpointer data)
+{
+    gint x, y;
+
+    gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
+    SPI_generateMouseEvent (x, y, "b1r");
+
+    return FALSE;
+}
+
 static void
 dwell_do_pointer_click (MTClosure *mt, gint x, gint y)
 {
@@ -68,7 +79,11 @@ dwell_do_pointer_click (MTClosure *mt, gint x, gint y)
 
     switch (clicktype) {
     case DWELL_CLICK_TYPE_SINGLE:
-	SPI_generateMouseEvent (x, y, "b1c");
+	SPI_generateMouseEvent (x, y, "b1p");
+	/* Wait a few msecs before releasing the button again.
+	 * This allows GOK to see our clicks.
+	 */
+	g_timeout_add (60, dwell_b1r_timeout, NULL);
 	break;
     case DWELL_CLICK_TYPE_DOUBLE:
 	SPI_generateMouseEvent (x, y, "b1d");
@@ -96,13 +111,14 @@ dwell_do_pointer_click (MTClosure *mt, gint x, gint y)
 }
 
 static inline gboolean
-within_tolerance (MTClosure *mt, gint x, gint y)
+below_threshold (MTClosure *mt, gint x, gint y)
 {
-    gdouble distance;
+    gint dx, dy;
 
-    distance = sqrt(pow(x - mt->pointer_x, 2.) + pow(y - mt->pointer_y, 2.));
+    dx = x - mt->pointer_x;
+    dy = y - mt->pointer_y;
 
-    return distance < mt->threshold;
+    return (dx * dx + dy * dy) < (mt->threshold * mt->threshold);
 }
 
 static gboolean
@@ -110,10 +126,10 @@ analyze_direction (MTClosure *mt, gint x, gint y)
 {
     gint gd, i, dx, dy;
 
-    dx = ABS(x - mt->pointer_x);
-    dy = ABS(y - mt->pointer_y);
+    dx = ABS (x - mt->pointer_x);
+    dy = ABS (y - mt->pointer_y);
 
-    if (within_tolerance (mt, x, y))
+    if (below_threshold (mt, x, y))
 	return FALSE;
 
     /* find direction */
@@ -265,7 +281,7 @@ spi_motion_event (const AccessibleEvent *event, void *data)
     MTClosure *mt = data;
 
     if (mt->dwell_enabled) {
-	if (!within_tolerance (mt, event->detail1, event->detail2) &&
+	if (!below_threshold (mt, event->detail1, event->detail2) &&
 	    !mt->dwell_gesture_started) {
 	    mt->pointer_x = (gint) event->detail1;
 	    mt->pointer_y = (gint) event->detail2;
@@ -278,12 +294,12 @@ spi_motion_event (const AccessibleEvent *event, void *data)
 	    draw_line (mt->pointer_x, mt->pointer_y,
 		       event->detail1, event->detail2);
 
-	    mt->x_old = event->detail1;
-	    mt->y_old = event->detail2;
+	    mt->x_old = (gint) event->detail1;
+	    mt->y_old = (gint) event->detail2;
 	}
     }
     if (mt_timer_is_running (mt->delay_timer)) {
-	if (!within_tolerance (mt, event->detail1, event->detail2)) {
+	if (!below_threshold (mt, event->detail1, event->detail2)) {
 	    mt_timer_stop (mt->delay_timer);
 	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 	}
@@ -347,7 +363,7 @@ cursor_overlay_time (guchar  *image,
     return TRUE;
 }
 
-void
+static void
 mt_update_cursor (MtCursor *cursor, MtTimer *timer, gdouble time)
 {
     guchar *image;
@@ -532,8 +548,10 @@ accessibility_enabled (MTClosure *mt,
 	}
 	else {
 	    /* reset the selected option again */
-	    gconf_client_set_bool (mt->client, OPT_DELAY, FALSE, NULL);
-	    gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
+	    if (gconf_client_get_bool (mt->client, OPT_DELAY, NULL))
+		gconf_client_set_bool (mt->client, OPT_DELAY, FALSE, NULL);
+	    if (gconf_client_get_bool (mt->client, OPT_DWELL, NULL))
+		gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
 	}
 	return FALSE;
     }
@@ -609,7 +627,7 @@ main (int argc, char **argv)
 	{"enable-secondary", 0, 0, G_OPTION_ARG_NONE, &delay_click,
 	    _("Enable simulated secondary click"), 0},
 	{"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &dwell_time,
-	    _("Time to wait before a dwell click"), "[0.5-3.0]"},
+	    _("Time to wait before a dwell click"), "[0.2-3.0]"},
 	{"secondary-time", 0, 0, G_OPTION_ARG_DOUBLE, &delay_time,
 	    _("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
 	{"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &mode,
@@ -669,6 +687,7 @@ main (int argc, char **argv)
 	MtCursorManager *manager;
 	AccessibleEventListener *bl, *ml;
 	gint spi_status;
+	gint spi_leaks = 0;
 
 	if (mt_pidfile_create () < 0)
 	    return 1;
@@ -706,7 +725,7 @@ main (int argc, char **argv)
 	    gconf_client_set_bool (mt->client, OPT_DELAY, TRUE, NULL);
 	if (delay_time >= .5 && delay_time <= 3.)
 	    gconf_client_set_float (mt->client, OPT_DELAY_T, delay_time, NULL);
-	if (dwell_time >= .5 && dwell_time <= 3.)
+	if (dwell_time >= .2 && dwell_time <= 3.)
 	    gconf_client_set_float (mt->client, OPT_DWELL_T, dwell_time, NULL);
 	if (threshold >= 0 && threshold <= 30)
 	    gconf_client_set_int (mt->client, OPT_THRESHOLD, threshold, NULL);
@@ -747,11 +766,14 @@ main (int argc, char **argv)
 	    AccessibleEventListener_unref (bl);
 	    SPI_deregisterGlobalEventListenerAll (ml);
 	    AccessibleEventListener_unref (ml);
-	    SPI_exit ();
+	    spi_leaks = SPI_exit ();
 	    mt_closure_free (mt);
 	FINISH:
 	    mt_pidfile_remove ();
 	    g_object_unref (program);
+
+	if (spi_leaks)
+	    g_warning ("AT-SPI reported %i leaks", spi_leaks);
     }
     return 0;
 }
